# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QLayout

from ....io import IFilterHints, IoManager, IIoManagerUi, IStorageLocation
from ....model import adapt, DelegatedTo, Instance
from ....ui import IStorageLocationEditor

from ..editor_layout_adapter import EditorLayoutAdapter

from .location_editor_components import LocationEditorComponents


@adapt(QLayout, to=[IStorageLocationEditor, IFilterHints])
class QLayoutIStorageLocationEditorAdapter(EditorLayoutAdapter, LocationEditorComponents):
    """ An adapter to implement IStorageLocationEditor for a storage location
    editor.
    """

    # The filter hints.
    filter = DelegatedTo('filter_hints')

    # Remember the storage location between clicks of the browse button.
    _default_location = Instance(IStorageLocation)

    @classmethod
    def isadaptable(cls, adaptee):
        """ Determine if this adapter can adapt a particular layout.

        :param adaptee:
            is the layout.
        :return:
            ``True`` if the layout can be adapted.
        """

        return cls.usable_editor(adaptee)

    def configure(self, properties):
        """ Configure the editor. """

        line_edit = self.line_edit

        # For automation purposes give the line edit the id if it doesn't
        # already have one.
        if line_edit.objectName() == '':
            line_edit.setObjectName(self.id)

        # Connect things up.
        line_edit.textEdited.connect(self.qt4_notify)
        self.browse_button.clicked.connect(self._on_clicked)

        # Apply any properties to the line edit.
        try:
            line_edit.pyqtConfigure(**properties)
        except AttributeError:
            pass

    @IStorageLocationEditor.read_only.getter
    def read_only(self):
        """ Invoked to get the read-only state. """

        return self.line_edit.isReadOnly()

    @read_only.setter
    def read_only(self, value):
        """ Invoked to set the read-only state. """

        self.line_edit.setReadOnly(value)
        self.browse_button.setVisible(not value)

    @IStorageLocationEditor.value.getter
    def value(self):
        """ Invoked to get the editor's value. """

        return self.line_edit.text()

    @value.setter
    def value(self, value):
        """ Invoked to set the editor's value. """

        # Save for the notifier.
        self.qt4_old_value = value

        self.line_edit.setText(value)

    def set_focus(self):
        """ Give the focus to the line edit. """

        self.line_edit.setFocus()

    def _on_clicked(self):
        """ Invoked when the browse button is clicked. """

        iiomanagerui = IIoManagerUi(IoManager.ui)

        if self.mode == 'open':
            location = iiomanagerui.readable_storage_location(
                    self.window_title if self.window_title != '' else "Open",
                    default_location=self._default_location,
                    format=self.format, hints=self, parent=self.browse_button)
        else:
            location = iiomanagerui.writeable_storage_location(
                    self.window_title if self.window_title != '' else "Save",
                    default_location=self._default_location,
                    format=self.format, hints=self, parent=self.browse_button)

        self.line_edit.activateWindow()
        self.line_edit.setFocus()

        if location is not None:
            self._default_location = location
            self.value = str(location)
